import pygame
from OpenGL.GL import *
from OpenGL.GLU import *
from texture import *

class Engine:

    def initGL(self,surface):
        
        w = surface.get_width()
        h = surface.get_height()

        kx = w // self.XRES
        ky = h // self.YRES
        
        if kx<ky:
            if kx > 1: k = kx
            else: k = 1
        else:
            if ky > 1: k = ky
            else: k = 1
            
        glViewport((w-self.XRES*k)//2,(h-self.YRES*k)//2,
            self.XRES*k,self.YRES*k)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glTranslatef(-1.0,1.0,0.0)
        glScalef(2.0/self.XRES,-2.0/self.YRES,1.0)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        glEnable(GL_TEXTURE_2D)
        glEnable(GL_ALPHA_TEST)
        glAlphaFunc(GL_EQUAL,GL_ONE)
        glClearColor(0.3,0.3,0.3,1.0)
        glPointSize(3.0);
        
        if self.fontex != None:
             self.regeneratefont()
            
        for t in self.textures.itervalues():
            t.regenerate()
        
    def line(self,frm,to,col):
        glColor3fv(col)
        self.whitex.bindme()
        glBegin(GL_LINES)
        glVertex2fv(frm)
        glVertex2fv(to)
        glEnd()
        
    def quad(self,quad,col):
        glColor3fv(col)
        self.whitex.bindme()
        glBegin(GL_QUADS)
        for q in quad:
            glVertex2fv(q)
        glEnd()
        
        
    def point(self,pos,col):
        glColor3fv(col)
        self.whitex.bindme()
        glBegin(GL_POINTS)
        glVertex2fv(pos)
        glEnd()

    def __init__(self, XRES, YRES,zoom):

        self.XRES = XRES
        self.YRES = YRES
        self.zoom = zoom
        self.framecount = 0
        self.fullscreen = False
        self.inputs = {}
        self.fcfunc = []
        self.textures = {}
        self.fontex = None
        
        pygame.init()
        self.initGL(
            pygame.display.set_mode((self.XRES*zoom,self.YRES*zoom),
            pygame.OPENGL|pygame.RESIZABLE|pygame.DOUBLEBUF))


    def setfont(self, fname, pitch):
        
        font = pygame.font.Font(fname,pitch)

        chars = []
        for i in range(32,127): chars.append(chr(i))
        s = ''.join(chars)

        self.fontmtr = font.metrics(s)
    
        fontsrc = font.render(s, False, (255,255,255))
        self.fontex = Texture(fontsrc,(0,0))
        self.fontheight = float(font.get_linesize())
        self.regeneratefont()


    def regeneratefont(self):

        self.fontex.regenerate()
        xp = 0.0
        yp = self.fontheight
        size = self.fontex.getsize()

        self.fontUV = []
        for i in range(0,95):
            
            U0 = xp/size[0]
            U = (xp+self.fontmtr[i][4])/size[0]
            V = -yp/size[1]
            W = self.fontmtr[i][4]
            H = yp
            
            list = glGenLists(1)
            glNewList(list,GL_COMPILE)
            glBegin(GL_QUADS)
            glTexCoord2f(U0, 0.0); glVertex2f( 0.0,  0.0)
            glTexCoord2f(U, 0.0); glVertex2f( W, 0.0)   
            glTexCoord2f(U, V); glVertex2f( W, H)   
            glTexCoord2f(U0, V); glVertex2f( 0.0, H)   
            glEnd()
            glEndList()
            self.fontUV.append((list,W))
                    
            xp += self.fontmtr[i][4]

    
    def getoffset(self, name):
        
        return self.textures[name].offset
    
    def getbitmask(self, name):
        
        return self.textures[name].bitmask
    
    def addtexture(self, name, surface, offset):
        
        t = Texture(surface, offset)
        t.regenerate()
        self.textures[name] = t

    def shadowtext(self, text, pos, fore, back):
    
        self.text(text,(pos[0],pos[1]+1),back)
        self.text(text,pos,fore)

    def outlinetext(self, text, pos, fore, back):
    
        self.text(text,(pos[0],pos[1]),back)
        self.text(text,(pos[0]+1,pos[1]),back)
        self.text(text,(pos[0]+2,pos[1]),back)
        self.text(text,(pos[0],pos[1]+1),back)
        self.text(text,(pos[0]+2,pos[1]+1),back)
        self.text(text,(pos[0],pos[1]+2),back)
        self.text(text,(pos[0]+1,pos[1]+2),back)
        self.text(text,(pos[0]+2,pos[1]+2),back)
        
        self.text(text,(pos[0]+1,pos[1]+1),fore)

    def filltext(self, text, pos, fore, back):
    
        glDisable(GL_TEXTURE_2D)
        self.text(text,pos,back)
        glEnable(GL_TEXTURE_2D)
        self.text(text,pos,fore)

    def image(self, name, pos):
        
        x = pos[0]
        y = pos[1]
        glPushMatrix()
        glTranslatef(x,y,0)
        self.textures[name].render()
        glPopMatrix()
        
    def imageref(self, name, pos, poseref):
        
        p1 = self.getoffset(name)
        p0 = self.getoffset(poseref)
            
        dx = p1[0] - p0[0]
        dy = p1[1] - p0[1]
          
        x = pos[0]
        y = pos[1]
        
        self.image(name, (x+dx,y+dy))

    def text(self, text, pos, fore):

        x = pos[0]
        y = pos[1]
        
        self.fontex.bindme()
        glColor3fv(fore)

        for c in text:

            if ord(c)==10:
                y+=self.fontheight
                x = pos[0]
            else:

                UV = self.fontUV[ord(c)-32]
                
                glPushMatrix()
                glTranslatef(x,y,0)
                glCallList(UV[0])
                glPopMatrix()

                x += UV[1]

    def inputcreate(self, id, defaultkey):
        self.inputs[id]=[[],False,defaultkey]

    def inputcallback(self, id, func):
        self.inputs[id][0].append(func)
    
    def inputkillback(self, id, func):
        self.inputs[id][0].remove(func)
        
    def framekillback(self, func):
        self.fcfunc.remove(func)
        
    def framecallback(self, func):
        self.fcfunc.append(func)
        
    def getframe(self):
        return self.framecount

    def pressed(self, id):
        return self.inputs[id][1]

    def start(self):
        
        pygame.time.set_timer(pygame.USEREVENT+1, 30)
        while 1:
            event = pygame.event.wait()
            if event.type == pygame.VIDEORESIZE:
                self.initGL(pygame.display.set_mode((event.w,event.h),
                    pygame.OPENGL|pygame.RESIZABLE|pygame.DOUBLEBUF))
            elif event.type == pygame.USEREVENT+1:
                for f in self.fcfunc:
                    f(self.framecount)
                glFlush()       
                pygame.display.flip()
                self.framecount = self.framecount + 1
            elif event.type == pygame.QUIT:
                break
            elif event.type == pygame.KEYDOWN:
                for k, v in self.inputs.iteritems():
                    if v[2] == event.key:
                        self.inputs[k][1] = True
                        for f in self.inputs[k][0]: f(True)
                
                if event.key == pygame.K_ESCAPE:
                    break
                elif event.key == pygame.K_F12:
                    self.fullscreen = not self.fullscreen
                    if self.fullscreen:
                        self.initGL(
                            pygame.display.set_mode((0,0),
                            pygame.OPENGL|pygame.FULLSCREEN|pygame.DOUBLEBUF))
                    else:                
                        self.initGL(
                            pygame.display.set_mode(
                            (self.XRES*self.zoom,self.YRES*self.zoom),
                            pygame.OPENGL|pygame.RESIZABLE|pygame.DOUBLEBUF))
                            
            elif event.type == pygame.KEYUP:
                for k, v in self.inputs.iteritems():
                    if v[2] == event.key:
                        self.inputs[k][1] = False
                        for f in self.inputs[k][0]: f(False)
