import math
from engine import *
import numpy
import os
import random


mobs = {"efx":[],"ffire":[],"player":[],"enemy":[],"efire":[]}
eng = Engine(320,240,2)
points = 0


class Alarm:
    
    def __init__(self, x, y, dx, dy):

        ringsound.play()
        self.x = x
        self.y = y
        self.dx = dx
        self.dy = dy
        self.pose = "allarme_1"
        self.base = "allarme_0"
        self.invulnerable = False
        
        eng.framecallback(self.frame)
        mobs["efire"].append(self)

    def remove(self):      
        eng.framekillback(self.frame)
        mobs["efire"].remove(self)
        
    def frame(self, n):
                     
        self.x = self.x + self.dx * 4
        self.y = self.y + self.dy * 4
        
        if self.x < -10 or self.y < -10 or self.x > 330 or self.y > 250:
            self.remove()

    def collision(self, mob):       
        self.remove()
               

class Lazor:
    
    def __init__(self, x, y, dx, dy):

        shotsound.play()
        self.x = x
        self.y = y
        self.base = "lazer_0"
        self.invulnerable = False
        
        if dx == 1:
            if dy == 1: self.pose="lazer_SE"
            elif dy == -1: self.pose="lazer_NE"
            else: self.pose="lazer_E"
        elif dx == -1: 
            if dy == 1: self.pose="lazer_SO"
            elif dy == -1: self.pose="lazer_NO"
            else: self.pose="lazer_O"           
        else:
            if dy == 1: self.pose="lazer_S"
            elif dy == -1: self.pose="lazer_N"
            else: 
                self.pose="lazer_E"
                dx = 1
           
        self.dx = dx
        self.dy = dy
        
        eng.framecallback(self.frame)
        mobs["ffire"].append(self)
       
    def remove(self):      
        eng.framekillback(self.frame)
        mobs["ffire"].remove(self)
             
    def frame(self, n):
       
        self.x = self.x + self.dx * 8
        self.y = self.y + self.dy * 8
        
        if self.x < -10 or self.y < -10 or self.x > 330 or self.y > 250:
            self.remove()

    def collision(self, mob):       
        self.remove()
        
       

class Clock:
    
    def __init__(self, shootrate, velchange):
        
        self.vx = -1
        self.vy = 0
        self.x = 340
        self.y = random.randint(40,200)
        self.base = "sveglia_0"
        self.pose = "sveglia_1"
        self.invulnerable = False
        self.invuln = 0
        self.shootrate = shootrate
        self.velchange = velchange
        
        eng.framecallback(self.frame)
        mobs["enemy"].append(self)

    def remove(self):      
        eng.framekillback(self.frame)
        mobs["enemy"].remove(self)

    def frame(self, n):
        
        if ((n/20)%2)==1:
            self.pose = "sveglia_1"
        else:
            self.pose = "sveglia_2"

        if self.invulnerable:
            if self.invuln < eng.getframe():
                self.remove()    
        else:
            if self.shootrate>0 and random.randint(0,self.shootrate)==0:    
                dx = (cat.x-self.x)
                dy = (cat.y-self.y)         
                dist = math.sqrt((dx*dx)+(dy*dy))         
                Alarm(self.x,self.y, dx/dist, dy/dist)
            if self.velchange>0 and random.randint(0,self.velchange)==0:
                self.vx = random.randint(-2,-1)
                self.vy = random.randint(-2,2)
                
                
        self.x = self.x + self.vx
        self.y = self.y + self.vy
        
        if self.x < -30 or self.y < -30 or self.x > 350 or self.y > 270:
            self.remove()
        
        
    def collision(self, mob):
        
        if not self.invulnerable:
            boomsound.play()
            global points 
            points = points + 10        
            self.invulnerable = True
            self.invuln = eng.getframe() + 30



class Cat:
    
    def __init__(self):
        
        self.lives = 9
        self.jumping = False
        self.crouching = False
        self.invulnerable = False
        self.dead = False
        self.invuln = 0
        
        self.x = 40
        self.y = 220
        self.jumpbig = []
        self.jumpsmall = []
        self.jumpstart = 0
        self.pose = "gatto_0"
        self.base = "gatto_0"
        
        for i in range(-20,20): 
            self.jumpbig.append(60+(i*i)*(220-60)/(20*20))
            self.jumpsmall.append(120+(i*i)*(220-120)/(20*20))
        
        eng.inputcallback("UP", self.jump)
        eng.inputcallback("FIRE", self.fire)
        eng.framecallback(self.frame)
        mobs["player"].append(self)

    def remove(self):      
        eng.framekillback(self.frame)
        mobs["player"].remove(self)
             
    def fire(self, pressed):
                        
        if pressed:
            
            dx = 0
            if eng.pressed("LEFT"): dx = dx - 1 
            if eng.pressed("RIGHT"): dx = dx + 1
            dy = 0
            if eng.pressed("UP"): dy = dy - 1 
            if eng.pressed("DOWN"): dy = dy + 1
            
            eyes = eng.getoffset(self.pose.replace("gatto","fire"))
            base = eng.getoffset("gatto_0")
            
            Lazor( 
                          self.x + eyes[0] - base[0], 
                          self.y + eyes[1] - base[1], 
                          dx, dy)                        
    
    def jump(self, pressed):
        
        if pressed and not self.jumping:
            self.jumping = True
            if self.crouching: self.jumpcycle = self.jumpbig
            else: self.jumpcycle = self.jumpsmall
            self.crouching = False
            self.jumpstart = eng.framecount
       
    def collision(self, mob):
        
        if not (self.invulnerable or self.dead):
            
            hitsound.play()
            self.lives = self.lives - 1
            if self.lives == 0:
                self.dead = True
                self.pose = "gatto_5"           
                eng.inputkillback("UP", self.jump)
                eng.inputkillback("FIRE", self.fire)
            else:  
                self.invulnerable = True
                self.invuln = eng.getframe() + 60
            
            
                                       
    def frame(self, n):
        
        if self.dead:
            self.pose = "gatto_5"
        else:
            if self.jumping:
                self.pose = "gatto_1"
            elif self.crouching:
                self.pose = "gatto_4"
            else:
                if ((n/6)%2)==1:
                    self.pose = "gatto_2"
                else:
                    self.pose = "gatto_3"

        if self.invulnerable and self.invuln < eng.getframe():
            self.invulnerable = False

        if self.jumping:          
            d = n-self.jumpstart
            if d<len(self.jumpcycle): self.y = self.jumpcycle[n-self.jumpstart] 
            else: self.jumping = False         
        else:
            self.y = 220          
            if self.dead:
                self.crouching = True
            else:
                self.crouching = eng.pressed("DOWN")
         
        if not self.crouching and not self.dead:
            if eng.pressed("LEFT"): self.x = self.x - 2
            if eng.pressed("RIGHT"): self.x = self.x + 2
        else:
            self.x = self.x - 1
            
        if self.x < 20:
            if self.dead:
                self.remove()
            else:
                self.crouching = False
                self.x = 20
        if self.x > 300: self.x = 300
     
     
def collide(m1,m2):

    p1 = eng.getoffset(m1.pose)
    p2 = eng.getoffset(m2.pose)
    b1 = eng.getoffset(m1.base)
    b2 = eng.getoffset(m2.base)
    
    return not (eng.getbitmask(m1.pose).overlap(eng.getbitmask(m2.pose),
                                         ( - (int(m1.x)+p1[0]-b1[0]) + (int(m2.x)+p2[0]-b2[0]),
                                           - (int(m1.y)+p1[1]-b1[1]) + (int(m2.y)+p2[1]-b2[1]) )) == None)
                       
def collisions(class1,class2):

    for c1 in mobs[class1]:
        for c2 in mobs[class2]:
            if collide(c1, c2):
                c1.collision(c2)
                c2.collision(c1)

def frame(n):
        
    collisions("player","efire")       
    collisions("enemy","ffire")       
    collisions("player","enemy")       
     
    WIN = False
     
    # RESPAWNING
    
    if points>=500:
        WIN = True     
    elif points>=400:
        while len(mobs["enemy"])<7: Clock(100,100)
    elif points>=300:
        while len(mobs["enemy"])<6: Clock(200,200)
    elif points>=200:
        while len(mobs["enemy"])<5: Clock(200,300)
    elif points>=150:
        while len(mobs["enemy"])<4: Clock(400,400)
    elif points>=100:
        while len(mobs["enemy"])<3: Clock(400, 500)
    elif points>=50:
        while len(mobs["enemy"])<2: Clock(0,600)
    else:
        while len(mobs["enemy"])<1: Clock(0,0)
    
    
    # RENDER PHASE
          
    glClear(GL_COLOR_BUFFER_BIT)
       
    eng.image("sfondo",(0-(n%320),0))
    eng.image("sfondo",(320-(n%320),0))
    
    if not WIN:             
        for ls in mobs.itervalues():
            for mob in ls:
                if not (mob.invulnerable and (n%2)==0):
                    eng.imageref(mob.pose,(mob.x,mob.y),mob.base)
    
    eng.outlinetext("A CAT'S DREAM - Use Cursors and Space - crouch to jump higher",(25.0,0.0),(0.7,0.7,0.7),(0.0,0.2,0.0))
    eng.outlinetext("Lives: " + str(cat.lives),(10.0,230.0),(0.3,0.3,0.3),(1.0,0.5,0.5))
    eng.outlinetext("Points: " + str(points),(160.0,230.0),(0.3,0.3,0.3),(0.5,0.5,1.0))

    if cat.lives==0:
        eng.outlinetext("GAME OVER, YOU SUCK!",(120.0,110.0),(1.0,1.0,1.0),(0.0,0.0,0.0))
        
        gamesong.stop()
        diesong.play()
        
    if WIN:
        eng.outlinetext("GAME OVER, YOU WIN!",(120.0,110.0),(1.0,1.0,1.0),(0.0,0.0,0.0))
        eng.imageref("gatto_2",(50,220),"gatto_0")
        eng.imageref("cake_1",(160,220),"cake_0")
        
        gamesong.stop()
        winsong.play()
        
        eng.framekillback(frame)
        

eng.setfont('../data/04B_03__.TTF',8)
layers = open("../data/layers.txt","rt")
        
layers.readline()
layers.readline()
        
nlayers = int(layers.readline())
        
while nlayers>=0:
    s = pygame.image.load(open("../data/" + layers.readline().rstrip(),"rb"))
    name = layers.readline().rstrip()
    layers.readline()
    x = int(layers.readline())
    y = int(layers.readline())
    layers.readline()
    layers.readline()
    layers.readline()
            
    eng.addtexture(name,s,(x,y))
    nlayers = nlayers - 1;
                                                              
eng.inputcreate("LEFT",pygame.K_LEFT)
eng.inputcreate("RIGHT",pygame.K_RIGHT)
eng.inputcreate("UP",pygame.K_UP)
eng.inputcreate("DOWN",pygame.K_DOWN)
eng.inputcreate("FIRE",pygame.K_SPACE)
eng.framecallback(frame)
 
cat = Cat()

pygame.mixer.init(44100,16,2,256)
winsong = pygame.mixer.Sound("../data/win.ogg")
diesong = pygame.mixer.Sound("../data/die.ogg")
gamesong = pygame.mixer.Sound("../data/ingame.ogg")
hitsound = pygame.mixer.Sound("../data/hit.ogg")
shotsound = pygame.mixer.Sound("../data/shot.ogg")
boomsound = pygame.mixer.Sound("../data/boom.ogg")
ringsound = pygame.mixer.Sound("../data/ring.ogg")

gamesong.play(-1)

eng.start()

